import logging
import threading

from hijiki.config.consumer_data import ConsumerData
from hijiki.adapters.rabbitmq_adapter import RabbitMQAdapter
from hijiki.connection.rabbitmq_connection import ConnectionParameters


class ConsumerRabbitMQAdapter(RabbitMQAdapter):
    def __init__(self, connection_params: ConnectionParameters, consumer_data: ConsumerData):
        super().__init__(connection_params)
        self.queue = consumer_data.queue
        self.topic = consumer_data.topic or consumer_data.queue
        self.handler = consumer_data.handler
        self.auto_ack = consumer_data.auto_ack
        self.routing_key = consumer_data.routing_key if consumer_data.routing_key else "*"
        self._consumer_thread = None
        self.queue_type = consumer_data.queue_type
        self.exchange_type = consumer_data.exchange_type
        self.create_dlq = consumer_data.create_dlq
        self.dlq_name = consumer_data.dlq_name
        self.dlx_name = consumer_data.dlx_name
        self.custom_queue_args = consumer_data.custom_queue_args


    def create_exchange_and_queue(self):
        channel = self.get_channel()
        default_queue_type = "quorum"
        default_queue_args_by_type = {
            "quorum": { "x-delivery-limit": 10 },
            "classic": {},
            "stream": {},
        }
        queue_type = self.queue_type or default_queue_type
        queue_args = (
            self.custom_queue_args
            if self.custom_queue_args is not None
            else default_queue_args_by_type.get(queue_type, {})
        )
        queue_args["x-queue-type"] = queue_type

        default_exchange_type = "topic"
        exchange_type = self.exchange_type or default_exchange_type

        if self.create_dlq:
            if self.dlq_name:
                dlq_queue = self.dlq_name
            else:
                dlq_queue = f"{self.queue}_dlq" if not self.queue.endswith("_dlq") else self.queue

            if self.dlx_name:
                dlq_exchange = self.dlx_name
            else:
                dlq_exchange = f"{self.queue}_dlq_event" if not self.queue.endswith("_dlq") else f"{self.queue}_event"

            channel.exchange_declare(exchange=dlq_exchange, exchange_type=exchange_type, durable=True)
            channel.queue_declare(queue=dlq_queue, durable=True, arguments=queue_args)
            channel.queue_bind(queue=dlq_queue, exchange=dlq_exchange, routing_key=self.routing_key)

            queue_args["x-dead-letter-exchange"] = dlq_exchange
            channel.queue_declare(queue=self.queue, durable=True, arguments=queue_args)
        else:
            channel.queue_declare(queue=self.queue, durable=True, arguments=queue_args)

        if self.topic:
            channel.exchange_declare(exchange=self.topic, exchange_type=exchange_type, durable=True)
            channel.queue_bind(queue=self.queue, exchange=self.topic, routing_key=self.routing_key)

    def _consume(self):
        def callback(ch, method, properties, body):
            try:
                if self.auto_ack:
                    logging.info(f"Mensagem recebida na fila {self.queue}: {body.decode()}")
                    ch.basic_ack(delivery_tag=method.delivery_tag)
                self.handler(body)
                if not self.auto_ack:
                    ch.basic_ack(delivery_tag=method.delivery_tag)
            except Exception as e:
                logging.error(f"Erro ao processar mensagem: {e}")
                if not self.auto_ack:
                    ch.basic_reject(delivery_tag=method.delivery_tag, requeue=True)

        try:
            self.get_channel().basic_consume(queue=self.queue, on_message_callback=callback)
            logging.info(f"Iniciando consumo na fila: {self.queue}")
            self.get_channel().start_consuming()
        except Exception as e:
            logging.error(f"Erro ao iniciar consumo com dados do adapter: {self.connection_data}")
            raise e

    def consume(self):
        self.connect()
        self.create_exchange_and_queue()
        """Inicia o consumo da fila em uma thread separada."""
        if self._consumer_thread and self._consumer_thread.is_alive():
            logging.warning("O consumidor já está rodando.")
            return
        self._consumer_thread = threading.Thread(target=self._consume, daemon=True)
        self._consumer_thread.start()
        logging.info(f"Thread de consumo iniciada para a fila: {self.queue}")
    

    def stop_consuming(self):
        def _stop_consuming():
            if self.get_channel().is_open:
                logging.info(f"Parando consumo da fila: {self.queue}")
                self.get_channel().stop_consuming()

        if self._consumer_thread and self._consumer_thread.is_alive():
            if self.rabbit_connection and self.rabbit_connection.is_open:
                self.rabbit_connection.add_callback_threadsafe(_stop_consuming)

            self._consumer_thread.join()
            super().stop_consuming()



import unittest
from unittest.mock import patch, Mock

import pika

from hijiki.config.consumer_data import ConsumerData
from hijiki.manager.message_manager_builder import MessageManagerBuilder

DLQ_RETRY_NUMsBER = 11 # At eleven interaction the message goes to the Dead Letter Queue
SECS_TO_AWAIT_BROKER = 5


class TestBrokerPing(unittest.TestCase):

    @patch("pika.BlockingConnection", spec=pika.BlockingConnection)
    def test_success_ping(self, mock_pika):
        mock_pika.return_value = Mock()
        mock_pika.is_open.return_value = True
        manager = (MessageManagerBuilder.get_instance() \
            .with_user("user") \
            .with_password("pwd") \
            .with_host("localhost") \
            .with_port(5672) \
            .build())
        consumer_data = ConsumerData("test_queue", "test_topic", handler=self.fail_ping_handler,
                                     dlq_handler=self.fail_ping_handler_dlq())
        manager.create_consumer(consumer_data)
        manager.start_consuming()

        for consumer in manager.broker.consumers.values():
            consumer.connection = mock_pika
        self.assertTrue(manager.is_alive())

    def fail_ping_handler(self):
        pass

    def fail_ping_handler_dlq(self):
        pass

    @patch("pika.BlockingConnection", spec=pika.BlockingConnection)
    def test_fail_ping(self, mock_pika):
        mock_pika.return_value = Mock()
        mock_pika.is_open.return_value = False
        manager = MessageManagerBuilder.get_instance() \
            .with_user("user") \
            .with_password("wrong_pwd") \
            .with_host("localhost") \
            .with_port(5672) \
            .build()
        consumer_data = ConsumerData("test_queue", "test_topic", handler=self.fail_ping_handler,
                                     dlq_handler=self.fail_ping_handler_dlq())
        manager.create_consumer(consumer_data)
        for consumer in manager.broker.consumers.values():
            consumer.connection = mock_pika
        self.assertFalse(manager.is_alive())




from hijiki.config.broker_config import BrokerConfig
from hijiki.config.broker_type import BrokerType
from hijiki.config.consumer_data import ConsumerData
from hijiki.manager.message_manager import MessageManager
from hijiki.adapters.rabbitmq_broker import RabbitMQBroker
from hijiki.connection.rabbitmq_connection import ConnectionParameters


class MessageManagerBuilder:
    _instance = None  # Instância única do MessageManagerBuilder

    def __init__(self, recreate=False):
        if MessageManagerBuilder._instance is not None:
            if not recreate:
                # Se for para recriar, limpa a instância anterior
                raise ValueError("Apenas uma instância do MessageManager pode existir.")
            else:
                # Se for para recriar, limpa a instância anterior
                MessageManagerBuilder._instance = None

        broker_config = BrokerConfig()
        self.host = broker_config.get_host()
        self.port = broker_config.get_port()
        self.virtual_host = None
        self.user = broker_config.get_user()
        self.password = broker_config.get_password()
        self.cluster_hosts = broker_config.get_cluster_hosts()
        self.consumers_data = []
        self.broker_type = BrokerType.RABBITMQ
        self.heartbeat_interval = 60 # 60 é o tempo, em segundos, padrão do RabbitMQ desde a versão 3.3.5
        self.use_secure_protocol = False
        self.manager = None
        MessageManagerBuilder._instance = self
        self.possible_consumers = []  # Lista de consumidores via decorator que podem ser registrados posteriormente.

    @staticmethod
    def get_instance(recreate=False):
        if MessageManagerBuilder._instance is None or recreate:
            MessageManagerBuilder(recreate)
        return MessageManagerBuilder._instance

    def with_host(self, host: str):
        self.host = host
        return self

    def with_port(self, port: int):
        self.port = port
        return self

    def with_virtual_host(self, virtual_host: str):
        self.virtual_host = virtual_host
        return self

    def with_user(self, user: str):
        self.user = user
        return self

    def with_password(self, password: str):
        self.password = password
        return self

    def with_cluster_hosts(self, cluster_hosts: str):
        self.cluster_hosts = cluster_hosts
        return self

    def with_consumers(self, consumers_data: list[ConsumerData]):
        self.consumers_data = consumers_data
        return self

    def with_broker_type(self, broker_type: BrokerType):
        self.broker_type = broker_type
        return self

    def with_heartbeat_interval(self, heartbeat_interval: int):
        self.heartbeat_interval = heartbeat_interval
        return self

    def with_secure_protocol(self, use_secure_protocol: bool):
        self.use_secure_protocol = use_secure_protocol
        return self

    def add_possible_consumer(self, consumer_data: ConsumerData):
        self.possible_consumers.append(consumer_data)

    def build(self) -> MessageManager:
        extra_params = {
            'heartbeat': self.heartbeat_interval
        }
        connection_params = ConnectionParameters(self.host, self.port, self.virtual_host, self.user, self.password, self.cluster_hosts, self.use_secure_protocol, extra_connection_params=extra_params)
        broker = RabbitMQBroker(connection_params) if self.broker_type == BrokerType.RABBITMQ else None
        if not broker:
            raise ValueError("BrokerType inválido ou não suportado.")

        if not MessageManagerBuilder._instance.manager:
            # Cria uma nova instância do MessageManager se não existir
            manager = MessageManager(broker)
            MessageManagerBuilder._instance.manager= manager
        else:
            MessageManagerBuilder._instance.manager.define_broker(broker)

        # Registra automaticamente todos os consumidores informados
        for consumer_data in self.possible_consumers:
            MessageManagerBuilder.get_instance().manager.create_consumer(consumer_data)

        # Registra automaticamente todos os consumidores informados
        for consumer_data in self.consumers_data:
            MessageManagerBuilder.get_instance().manager.create_consumer(consumer_data)

        return MessageManagerBuilder.get_instance().manager


from typing import Optional
from hijiki.config.consumer_data import ConsumerData

from abc import ABC, abstractmethod

class MessageBroker(ABC):
    @abstractmethod
    def publish(self, topic: str, message: str, routing_key: str = 'x', exchange_type: Optional[str] = "topic",
                reply_to: Optional[str] = None):
        pass

    @abstractmethod
    def create_consumer(self, consumer_data: ConsumerData):
        pass

    @abstractmethod
    def start_consuming(self):
        pass

    @abstractmethod
    def ping(self):
        pass


import logging
from urllib.parse import quote

import pika
from typing import Optional

from hijiki.config.broker_config import BrokerConfig


class ConnectionParameters:
    def __init__(self, host: str = None, port: int = None, virtual_host: str = None, user: str = None,
                 password: str = None, cluster_hosts: str="", use_secure_protocol: bool = False, extra_connection_params: Optional[dict] = None):
        self.host = host
        self.port = port
        self.virtual_host = virtual_host
        self.user = user
        self.password = password
        self.cluster_hosts = cluster_hosts
        self.use_secure_protocol = use_secure_protocol
        self.extra_connection_params = extra_connection_params or {}

    def __str__(self):
        return f"ConnectionParameters(host={self.host}, port={self.port}, virtual_host={self.virtual_host}, user={self.user}, password=****, cluster_hosts={self.cluster_hosts})"

class RabbitMQConnection:
    def __init__(self, connection_params: ConnectionParameters = None):
        """Inicializa a conexão com o RabbitMQ usando um objeto ConnectionParameters."""
        self.host = connection_params.host if connection_params and connection_params.host else BrokerConfig.get_host()
        self.port = connection_params.port if connection_params and connection_params.port else BrokerConfig.get_port()
        self.virtual_host = f"/{quote(connection_params.virtual_host, safe='')}" if connection_params and connection_params.virtual_host else ""
        self.user = connection_params.user if connection_params and connection_params.user else BrokerConfig.get_user()
        self.password = connection_params.password if connection_params and connection_params.password else BrokerConfig.get_password()
        self.cluster_hosts = connection_params.cluster_hosts if connection_params and connection_params.cluster_hosts else BrokerConfig.get_cluster_hosts()
        self.extra_connection_params = connection_params.extra_connection_params if connection_params and connection_params.extra_connection_params else {}
        self.use_secure_protocol = connection_params.use_secure_protocol if connection_params and connection_params.use_secure_protocol else False
        self.connection = None

    def __validate_host(self):
        if self.host and self.cluster_hosts:
            raise Exception("BROKER_HOST e BROKER_CLUSTER_SERVER são mutuamente exclusivos e não podem os dois estarem definidos")

    def _get_connecttion_url_params(self) -> str:
        result = "?"
        for key, value in self.extra_connection_params.items():
           result += f"{key}={value}&"
        return result[:-1]  # Remove o último '&'

    def get_broker_url(self) -> str:
        self.__validate_host()
        """Gera a URL de conexão para o broker."""
        url_connectio_params = self._get_connecttion_url_params()
        protocol = "amqps" if self.use_secure_protocol else "amqp"
        if self.cluster_hosts:
            cluster = self.cluster_hosts.split(",")
            urls = [f"{protocol}://{self.user}:{self.password}@{host}{self.virtual_host}{url_connectio_params}" for host in cluster]
            return ";".join(urls)
        else:
            return f"{protocol}://{self.user}:{self.password}@{self.host}:{self.port}{self.virtual_host}{url_connectio_params}"

    def connect(self):
        broker_url = self.get_broker_url()
        logging.info(f"Conectando ao RabbitMQ com a URL: {broker_url}")
        self.connection = pika.BlockingConnection(pika.URLParameters(broker_url))
        logging.info("Conectado ao RabbitMQ")
        return self.connection


    def ping(self):
        try:
            if self.connection and self.connection.is_open():
                self.connection.process_data_events()
                return True
        except Exception:
            return False
        return False


import logging
from typing import Optional

import pika  # Usando a biblioteca pika diretamente

from hijiki.adapters.rabbitmq_adapter import RabbitMQAdapter
from hijiki.connection.rabbitmq_connection import ConnectionParameters


class PublisherRabbitMQAdapter(RabbitMQAdapter):
    def __init__(self, connection_data: ConnectionParameters):
        super().__init__(connection_data)
        self.connect()

    def publish(self, topic: str, message: str, routing_key: Optional[str] = "x",
                exchange_type: Optional[str] = "topic", reply_to: Optional[str] = None):
        channel = self.get_channel()
        channel.exchange_declare(exchange=topic, exchange_type=exchange_type, durable=True)

        channel.basic_publish(
            exchange=topic,
            routing_key=routing_key,
            body=message,
            properties=pika.BasicProperties(delivery_mode=2, reply_to=reply_to)
        )
        logging.info(f"Mensagem enviada para o tópico {topic}: {message}")



import logging
from typing import Optional

from hijiki.config.consumer_data import ConsumerData
from hijiki.ports.message_broker import MessageBroker
from hijiki.adapters.rabbitmq_consumer_adapter import ConsumerRabbitMQAdapter
from hijiki.adapters.rabbitmq_publisher_adapter import PublisherRabbitMQAdapter
from hijiki.connection.rabbitmq_connection import RabbitMQConnection

class RabbitMQBroker(MessageBroker):
    def __init__(self, connection_params):
        self.publisher = None
        self.connection_params = connection_params
        self.consumers = {}

    def publish(self, topic: str, message: str, routing_key: str = 'x', exchange_type: Optional[str] = "topic",
                reply_to: Optional[str] = None):
        publisher = PublisherRabbitMQAdapter(self.connection_params)
        publisher.publish(topic, message, routing_key, exchange_type, reply_to)

    def create_consumer(self, consumer_data: ConsumerData):
        if consumer_data.handler:
            adapter = ConsumerRabbitMQAdapter(self.connection_params, consumer_data)
            self.consumers[consumer_data.queue] = adapter
            logging.info(f"Consumidor criado para a fila {consumer_data.queue} e tópico {consumer_data.topic or consumer_data.queue}")
            return adapter
        else:
            consumer_adapter = self.consumers[consumer_data.queue]
            consumer_adapter.queue_declare(queue=consumer_data.queue, durable=True)
            logging.info(f"Fila {consumer_data.queue} criada sem handler.")
            return None

    def start_consuming(self):
        """Inicia o consumo das filas registradas"""
        for consumer in self.consumers.values():
            consumer.consume()

    def ping(self):
        if not self.publisher and not len(self.consumers.values()):
            try:
                connection = RabbitMQConnection(self.connection_params)
                conn = connection.connect()
                conn.close()
                return True
            except Exception as e:
                logging.error(f"RabbitMQ ping failed: {e}")
                return False

        result = True
        for consumer in self.consumers.values():
            result = result and consumer.ping()
        publisher_result = self.publisher.ping() if self.publisher else True
        result = result and publisher_result
        return result



import os
import unittest
from hijiki.connection.rabbitmq_connection import RabbitMQConnection, ConnectionParameters


class TestRabbitMQConnection(unittest.TestCase):

    def setUp(self):
        self.env_vars = {
            'BROKER_SERVER': 'env_host',
            'BROKER_PORT': '5673',
            'BROKER_USERNAME': 'env_user',
            'BROKER_PWD': 'env_password',
            'BROKER_CLUSTER_SERVER': 'env_host1,env_host2'
        }
        self.original_env = {key: os.environ.get(key) for key in self.env_vars}
        os.environ.update(self.env_vars)

    def tearDown(self):
        for key, value in self.original_env.items():
            if value is None:
                if key in os.environ:
                    del os.environ[key]
            else:
                os.environ[key] = value

    def test_get_broker_url_from_env(self):
        os.environ.pop('BROKER_CLUSTER_SERVER')  # Garantir que apenas BROKER_SERVER está definido
        connection = RabbitMQConnection()
        expected_url = 'amqp://env_user:env_password@env_host:5673'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_from_env_with_cluster(self):
        os.environ.pop('BROKER_SERVER')  # Garantir que apenas BROKER_CLUSTER_SERVER está definido
        connection = RabbitMQConnection()
        expected_url = 'amqp://env_user:env_password@env_host1;amqp://env_user:env_password@env_host2'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_from_builder(self):
        os.environ.pop('BROKER_CLUSTER_SERVER')
        connection_parameters = ConnectionParameters(host='builder_host', port=5674, user='builder_user',
                           password='builder_password')
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://builder_user:builder_password@builder_host:5674'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_from_builder_with_cluster(self):
        os.environ.pop('BROKER_SERVER')  # Garantir que apenas BROKER_CLUSTER_SERVER está definido
        connection_parameters = ConnectionParameters(host=None, port=None, user=None, password=None,
                                        cluster_hosts='builder_host1,builder_host2')
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://env_user:env_password@builder_host1;amqp://env_user:env_password@builder_host2'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_from_builder_with_cluster_and_heartbeat(self):
        os.environ.pop('BROKER_SERVER')  # Garantir que apenas BROKER_CLUSTER_SERVER está definido
        connection_parameters = ConnectionParameters(host=None, port=None, user=None, password=None,
                                        cluster_hosts='builder_host1,builder_host2', extra_connection_params={"heartbeat":36})
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://env_user:env_password@builder_host1?heartbeat=36;amqp://env_user:env_password@builder_host2?heartbeat=36'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_from_builder_with_heartbeat(self):
        os.environ.pop('BROKER_CLUSTER_SERVER')
        connection_parameters = ConnectionParameters(host='builder_host', port=5674, user='builder_user',
                           password='builder_password', extra_connection_params={"heartbeat":37})
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://builder_user:builder_password@builder_host:5674?heartbeat=37'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_with_virtual_host(self):
        os.environ.pop('BROKER_CLUSTER_SERVER')
        connection_parameters = ConnectionParameters(virtual_host='builder_virtual_host')
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://env_user:env_password@env_host:5673/builder_virtual_host'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_with_properly_uri_encoded_virtual_host(self):
        os.environ.pop('BROKER_CLUSTER_SERVER')
        connection_parameters = ConnectionParameters(virtual_host='/')
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://env_user:env_password@env_host:5673/%2F'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_with_virtual_host_and_heartbeat(self):
        os.environ.pop('BROKER_CLUSTER_SERVER')
        connection_parameters = ConnectionParameters(virtual_host='builder_virtual_host', extra_connection_params={"heartbeat":37})
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://env_user:env_password@env_host:5673/builder_virtual_host?heartbeat=37'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_with_virtual_host_and_cluster(self):
        os.environ.pop('BROKER_SERVER')
        connection_parameters = ConnectionParameters(virtual_host='builder_virtual_host', cluster_hosts='builder_host1,builder_host2')
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://env_user:env_password@builder_host1/builder_virtual_host;amqp://env_user:env_password@builder_host2/builder_virtual_host'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_with_virtual_host_and_cluster_and_heartbeat(self):
        os.environ.pop('BROKER_SERVER')
        connection_parameters = ConnectionParameters(virtual_host='builder_virtual_host', cluster_hosts='builder_host1,builder_host2',
                                                     extra_connection_params={"heartbeat":37})
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqp://env_user:env_password@builder_host1/builder_virtual_host?heartbeat=37;amqp://env_user:env_password@builder_host2/builder_virtual_host?heartbeat=37'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_with_secure_protocol(self):
        os.environ.pop('BROKER_CLUSTER_SERVER')
        connection_parameters = ConnectionParameters(use_secure_protocol=True)
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqps://env_user:env_password@env_host:5673'
        self.assertEqual(connection.get_broker_url(), expected_url)

    def test_get_broker_url_with_secure_protocol_and_cluster(self):
        os.environ.pop('BROKER_SERVER')
        connection_parameters = ConnectionParameters(use_secure_protocol=True, cluster_hosts='builder_host1,builder_host2')
        connection = RabbitMQConnection(connection_parameters)
        expected_url = 'amqps://env_user:env_password@builder_host1;amqps://env_user:env_password@builder_host2'
        self.assertEqual(connection.get_broker_url(), expected_url)


import json
import time
import unittest

from tests.runner import Runner

DLQ_RETRY_NUMBER = 11 # At eleven interaction the message goes to the Dead Letter Queue
SECS_TO_AWAIT_BROKER = 5
DLQ_DONT_RECEIVE_ERROR_WITH_AUTO_ACK_ENABLED = 1

def default_message_mapper(event_name: str, data: str):
    return {"value": data}


class TestPublisherConsumer(unittest.TestCase):
    runner = None

    @classmethod
    def setUpClass(cls):
        cls.runner = Runner()
        cls.runner.run()
        cls.NUMBER_OF_QUEUED_MESSAGES = 2  # Já estava definido na linha anterior, só conferindo duplicidade
        cls.test_count = 0

    @classmethod
    def tearDownClass(cls):
        if cls.test_count != 10:
            raise Exception(f"Expected 10 tests, but found {cls.test_count}. Please check the test cases.")

    def tearDown(self):
        self.runner.clear_results()


    def test_publish_a_message(self):
        self.runner.clear_results()
        self.runner.publish_message('teste1_event', '{"value": "This is the message"}')
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.runner.publish_message('teste1_event', '{"value": "This is the message"}')
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(len(self.runner.get_results()), 1)
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message_with_other_mapper(self):
        self.runner.clear_results()
        def other_message_mapper(event_name: str, data: str):
            return {"other_key": data, "event_name": event_name}

        event_name = 'teste1_event'
        message = "This is the message"
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.runner.publish_message(event_name, message, message_mapper=other_message_mapper)
        time.sleep(SECS_TO_AWAIT_BROKER)
        data = self.runner.get_results_data()
        self.assertEqual(len(data), 1)
        json_data = json.loads(data[0])
        self.assertEqual(json_data['other_key'], message)
        self.assertEqual(json_data['event_name'], event_name)
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message_failed(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.runner.publish_message('erro_event', '{"value": "This is the message"}')
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(DLQ_RETRY_NUMBER, len(self.runner.get_results()))
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message_failed_with_auto_ack_dont_go_to_DLQ(self):
        self.runner.clear_results()
        self.runner.gr.consumers["fila_erro"].auto_ack = True
        try:
            time.sleep(SECS_TO_AWAIT_BROKER)
            self.runner.publish_message('erro_event', '{"value": "This is the message"}')
            time.sleep(SECS_TO_AWAIT_BROKER)
            self.assertEqual(DLQ_DONT_RECEIVE_ERROR_WITH_AUTO_ACK_ENABLED, len(self.runner.get_results()))
        finally:
            self.runner.gr.consumers["fila_erro"].auto_ack = False
        TestPublisherConsumer.test_count += 1


    def test_consume_a_message_dlq(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        for number in range(self.NUMBER_OF_QUEUED_MESSAGES):
            self.runner.publish_message(
                'erro_event',
                f'{{"value": "This is message number {number} that will be sent to dlq"}}'
            )

        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(self.NUMBER_OF_QUEUED_MESSAGES, len(self.runner.get_results_dlq()))
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message_without_consumer_dlq(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        for number in range(self.NUMBER_OF_QUEUED_MESSAGES):
            self.runner.publish_message(
                'without_dlq',
                '{"value": "This is the message that will fall into a dlq queue, which has no consumer"}'
            )
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(
            self.NUMBER_OF_QUEUED_MESSAGES * DLQ_RETRY_NUMBER,
            len(self.runner.get_results())
        )
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message_with_specific_routing_key(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.runner.publish_message('teste1_event', '{"value": "This is the message"}')
        self.runner.publish_message('teste1_event', '{"value": "This is the message"}', routing_key='specific_routing_key')
        time.sleep(SECS_TO_AWAIT_BROKER)
        print(self.runner.get_result_for_specific_routing_key())
        self.assertEqual(1, len(self.runner.get_result_for_specific_routing_key()))
        self.assertEqual(3, len(self.runner.get_results_data())) # são tres mensagens, pois a fila teste1_event recebe dois por causa do routing key coring "*"
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message_with_custom_exchange_and_queue_type(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.runner.publish_message('with_direct_exchange_event', '{"value": "This is the message"}',
                                    routing_key='direct_routing_key', exchange_type='direct')
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(1, len(self.runner.get_results_data()))
        TestPublisherConsumer.test_count += 1

    def test_consume_a_message_with_custom_dead_letter_names(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        for number in range(self.NUMBER_OF_QUEUED_MESSAGES):
            self.runner.publish_message(
                'with.custom.dl.names.event',
                f'{{"value": "This is message number {number} that will be sent to dlq"}}'
            )

        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(self.NUMBER_OF_QUEUED_MESSAGES, len(self.runner.get_results_dlq()))
        TestPublisherConsumer.test_count += 1


import unittest
from unittest.mock import Mock, patch

from hijiki.config.broker_type import BrokerType
from hijiki.config.consumer_data import ConsumerData
from hijiki.manager.message_manager import MessageManager
from hijiki.manager.message_manager_builder import MessageManagerBuilder


class TestMessageManagerBuilder(unittest.TestCase):

    @patch("hijiki.adapters.rabbitmq_adapter.RabbitMQAdapter", autospec=True)
    @patch("pika.BlockingConnection", spec=True)
    def test_build_throws_error_with_invalid_broker_type(self, mock_pika, mock_adapter):
        mock_adapter.return_value = Mock()
        mock_pika.return_value = Mock()
        builder = MessageManagerBuilder.get_instance().with_broker_type("INVALID_BROKER")
        with self.assertRaises(ValueError):
            builder.build()

    @patch("hijiki.adapters.rabbitmq_adapter.RabbitMQAdapter", autospec=True)
    def test_singleton_instance_logic(self, mock_adapter):
        mock_adapter.return_value = Mock()
        instance1 = MessageManagerBuilder.get_instance()
        instance2 = MessageManagerBuilder.get_instance()
        self.assertIs(instance1, instance2)

    @patch("hijiki.adapters.rabbitmq_adapter.RabbitMQAdapter", autospec=True)
    @patch("hijiki.connection.rabbitmq_connection.ConnectionParameters", spec=True)
    def test_with_broker_type_rabbitmq_creates_rabbitmq_broker(self, mock_connection_params, mock_adapter):
        mock_adapter.return_value = Mock()
        mock_connection_params.return_value = Mock()
        builder = MessageManagerBuilder.get_instance().with_broker_type(BrokerType.RABBITMQ)
        manager = builder.build()
        self.assertIsNotNone(manager.broker)
        self.assertEqual(manager.broker.__class__.__name__, "RabbitMQBroker")

    @patch("hijiki.adapters.rabbitmq_adapter.RabbitMQAdapter", autospec=True)
    @patch("hijiki.connection.rabbitmq_connection.ConnectionParameters", spec=True)
    def test_with_consumers_registers_all_consumers(self, mock_connection_params, mock_adapter):
        mock_adapter.return_value = Mock()
        mock_connection_params.return_value = Mock()

        consumer1 = ConsumerData(queue="queue1", topic="topic1", handler=Mock())
        consumer2 = ConsumerData(queue="queue2", topic="topic2", handler=Mock())

        builder = (
            MessageManagerBuilder.get_instance()
            .with_consumers([consumer1, consumer2])
            .with_broker_type(BrokerType.RABBITMQ)
        )
        manager = builder.build()

        self.assertIn("queue1", manager.consumers)
        self.assertIn("queue2", manager.consumers)

    @patch("hijiki.adapters.rabbitmq_adapter.RabbitMQAdapter", autospec=True)
    @patch("hijiki.manager.message_manager_builder.ConnectionParameters", spec=True)
    def test_with_virtual_host_uses_virtual_host_in_connection_params(self, mock_connection_params, mock_adapter):
        mock_adapter.return_value = Mock()
        mock_connection_params.return_value = Mock()

        builder = (
            MessageManagerBuilder.get_instance()
            .with_virtual_host("some_virtual_host")
        )
        manager = builder.build()

        call_args = mock_connection_params.call_args[0]
        virtual_host = call_args[2]
        self.assertEqual(virtual_host, "some_virtual_host")

    @patch("hijiki.adapters.rabbitmq_adapter.RabbitMQAdapter", autospec=True)
    @patch("hijiki.manager.message_manager_builder.ConnectionParameters", spec=True)
    def test_with_secure_protocol_uses_secure_protocol_in_connection_params(self, mock_connection_params, mock_adapter):
        mock_adapter.return_value = Mock()
        mock_connection_params.return_value = Mock()

        builder = (
            MessageManagerBuilder.get_instance()
            .with_secure_protocol(True)
        )
        manager = builder.build()

        call_args = mock_connection_params.call_args[0]
        virtual_host = call_args[6]
        self.assertEqual(virtual_host, True)

    @patch("hijiki.adapters.rabbitmq_adapter.RabbitMQAdapter", autospec=True)
    @patch("hijiki.connection.rabbitmq_connection.ConnectionParameters", spec=True)
    def test_recreate_builder_creates_new_instance_and_allows_new_manager_creation(self, mock_connection_params, mock_adapter):
        mock_adapter.return_value = Mock()
        mock_connection_params.return_value = Mock()

        builder1 = MessageManagerBuilder.get_instance().with_broker_type(BrokerType.RABBITMQ)
        manager1 = builder1.build()
        
        self.assertIsInstance(manager1, MessageManager)
        
        builder2 = MessageManagerBuilder.get_instance(recreate=True).with_broker_type(BrokerType.RABBITMQ)
        manager2 = builder2.build()
        
        self.assertIsInstance(manager2, MessageManager)
        self.assertIsNot(manager1, manager2)


import unittest
from unittest.mock import Mock

from hijiki.config.consumer_data import ConsumerData


class TestConsumerData(unittest.TestCase):
    def test_consumer_data_initialization(self):
        handler_mock = Mock()
        dlq_handler_mock = Mock()
        consumer_data = ConsumerData("test_queue", "test_topic", handler_mock, dlq_handler_mock, auto_ack=True)

        self.assertEqual(consumer_data.queue, "test_queue")
        self.assertEqual(consumer_data.topic, "test_topic")
        self.assertEqual(consumer_data.handler, handler_mock)
        self.assertEqual(consumer_data.dlq_handler, dlq_handler_mock)
        self.assertTrue(consumer_data.auto_ack)

    def test_consumer_data_with_routing_key(self):
        handler_mock = Mock()
        consumer_data = ConsumerData("test_queue", "test_topic", handler_mock, routing_key="specific_routing_key")

        self.assertEqual(consumer_data.queue, "test_queue")
        self.assertEqual(consumer_data.topic, "test_topic")
        self.assertEqual(consumer_data.routing_key, "specific_routing_key")
        self.assertTrue(consumer_data.create_dlq)

    def test_consumer_data_with_custom_queue_related_parameters(self):
        handler_mock = Mock()
        consumer_data = ConsumerData("test.queue", "test.exch", handler_mock, queue_type="classic",
                                     exchange_type="direct", dlq_name="test.dlq", dlx_name="test.dlx")

        self.assertEqual(consumer_data.queue, "test.queue")
        self.assertEqual(consumer_data.topic, "test.exch")
        self.assertEqual(consumer_data.queue_type, "classic")
        self.assertEqual(consumer_data.exchange_type, "direct")
        self.assertEqual(consumer_data.dlq_name, "test.dlq")
        self.assertEqual(consumer_data.dlx_name, "test.dlx")



from typing import Callable, Optional

class ConsumerData:
    def __init__(self, queue: str, topic: str, handler: Callable, dlq_handler: Optional[Callable] = None,
                 routing_key: Optional[str] = "*", auto_ack: Optional[bool] = False, create_dlq:bool = True,
                 queue_type: Optional[str] = None, exchange_type: Optional[str] = None, dlq_name: Optional[str] = None,
                 dlx_name: Optional[str] = None, custom_queue_args: Optional[dict] = None):
        self.queue = queue
        self.topic = topic
        self.handler = handler
        self.dlq_handler = dlq_handler
        self.auto_ack = auto_ack
        self.routing_key = routing_key
        self.create_dlq = create_dlq
        self.queue_type = queue_type
        self.exchange_type = exchange_type
        self.dlq_name = dlq_name
        self.dlx_name = dlx_name
        self.custom_queue_args = custom_queue_args


# message_manager.py
import json
from typing import Callable, Optional

from hijiki.ports.message_broker import MessageBroker
from hijiki.config.consumer_data import ConsumerData
import logging


def default_message_mapper(message: str, data: str):
    return {"value": data}

default_routing_key = 'x'


class MessageManager:
    def __init__(self, broker: MessageBroker):
        self.broker = broker
        self.consumers = {}
        self.possible_consumers = []

    def publish(self, topic: str, message: str, message_mapper: Optional[Callable[[str, str], dict]]=default_message_mapper,
                routing_key=default_routing_key, exchange_type: Optional[str] = "topic", reply_to: Optional[str] = None):
        if message_mapper:
            payload = message_mapper(topic, message)
        else:
            payload = default_message_mapper(topic, message)
        new_message = json.dumps(payload)
        self.broker.publish(topic, new_message, routing_key, exchange_type, reply_to)
        logging.info(f"Mensagem publicada no tópico {topic}: {message}")

    def create_consumer(self, consumer_data: ConsumerData):
        consumer_adapter = self.broker.create_consumer(consumer_data)
        self.consumers[consumer_data.queue] = consumer_adapter
        logging.info(f"Consumidor registrado para a fila {consumer_data.queue}")


    def start_consuming(self):
        self.broker.start_consuming()

    def stop_consuming(self):
        for consumer_data in self.consumers.values():
            if hasattr(consumer_data, 'stop_consuming') and callable(getattr(consumer_data, 'stop_consuming')):
                consumer_data.stop_consuming()

    def define_broker(self, broker: MessageBroker):
        """Define o broker a ser usado pelo MessageManager."""
        self.broker = broker
        logging.info(f"Broker definido: {broker}")

    def is_alive(self):
        try:
           return self.broker.ping()
        except Exception:
            return False


import logging

def configure_logger():
    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger()
    handler = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)


from typing import Callable, Optional

from hijiki.config.consumer_data import ConsumerData
from hijiki.manager.message_manager_builder import MessageManagerBuilder


def consumer_handler(queue_name: str, topic: Optional[str] = None, routing_key: Optional[str] = None,
                     create_dlq=True, exchange_type: Optional[str] = None, queue_type: Optional[str] = None,
                     dlq_name: Optional[str] = None, dlx_name: Optional[str] = None) -> Callable:
    def decorator(func: Callable):
        consumer_data = ConsumerData(
            queue=queue_name,
            topic=topic or f"{queue_name}_event",
            handler=func,
            routing_key=routing_key,
            create_dlq=create_dlq,
            exchange_type=exchange_type,
            queue_type=queue_type,
            dlq_name=dlq_name,
            dlx_name=dlx_name
        )
        manager_builder =  MessageManagerBuilder.get_instance()
        if manager_builder:
            manager_builder.add_possible_consumer(consumer_data)
        return func
    return decorator




import unittest
import logging

from hijiki.logger import configure_logger


class TestLoggerConfig(unittest.TestCase):
    def test_logger_configuration(self):
        configure_logger()
        logger = logging.getLogger()
        self.assertTrue(any(isinstance(handler, logging.StreamHandler) for handler in logger.handlers))


import time
import unittest
from unittest.mock import Mock

from hijiki.adapters.rabbitmq_broker import RabbitMQBroker
from hijiki.config.consumer_data import ConsumerData
from hijiki.connection.rabbitmq_connection import ConnectionParameters
from hijiki.manager.message_manager_builder import MessageManagerBuilder
from hijiki.ports.message_broker import MessageBroker
from hijiki.manager.message_manager import MessageManager
import json

SECS_TO_AWAIT_BROKER = 2

def _other_message_mapper(event_name: str, data: str):
    return {"other_key": data, "event_name": event_name}


class TestMessageManager(unittest.TestCase):
    def setUp(self):
        self.broker_mock = Mock(spec=MessageBroker)
        self.manager = MessageManager(self.broker_mock)

    def test_publish_message(self):
        topic = "test_topic"
        message = "test_message"
        self.manager.publish(topic, message)
        self.broker_mock.publish.assert_called_with(
            topic,
            json.dumps({'value': 'test_message'}),
            'x',
            'topic',
            None
        )

    def test_consume_a_message_with_other_mapper(self):
        event_name = 'teste1_event'
        message = "This is the message"
        self.manager.publish(event_name, message, message_mapper=_other_message_mapper)
        self.broker_mock.publish.assert_called_with(
            event_name,
            json.dumps(_other_message_mapper(event_name, message)),  # <- Aqui!
            'x',
            'topic',
            None
        )

    def test_create_consumer(self):
        queue = "test_queue"
        topic = "test_topic"
        handler_mock = Mock()
        consumer_data = ConsumerData(queue, topic, handler_mock)
        self.manager.create_consumer(consumer_data)
        self.assertIn(queue, self.manager.consumers)
        self.broker_mock.create_consumer.assert_called_with(consumer_data)

    def test_start_consuming(self):
        self.manager.start_consuming()
        self.broker_mock.start_consuming.assert_called()

    def test_stop_consuming_rabbitmq_running_consumer(self):
        connection_params = ConnectionParameters(
            host="localhost",
            port=5672,
            user="user",
            password="pwd"
        )
        actual_broker = RabbitMQBroker(connection_params)
        manager = MessageManager(actual_broker)

        consumer_data = ConsumerData("test_queue", "test_topic", lambda x: x)
        manager.create_consumer(consumer_data)

        manager.start_consuming()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertTrue(manager.is_alive())

        manager.stop_consuming()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertFalse(manager.is_alive())


import os
from hijiki.config.broker_env_vars import BrokerEnvVars

class BrokerConfig:
    @staticmethod
    def get_host() -> str:
        return os.getenv(BrokerEnvVars.BROKER_SERVER)

    @staticmethod
    def get_port() -> int:
        return int(os.getenv(BrokerEnvVars.BROKER_PORT)) if os.getenv(BrokerEnvVars.BROKER_PORT) else 5672

    @staticmethod
    def get_user() -> str:
        return os.getenv(BrokerEnvVars.BROKER_USERNAME)

    @staticmethod
    def get_password() -> str:
        return os.getenv(BrokerEnvVars.BROKER_PWD)

    @staticmethod
    def get_cluster_hosts() -> str:
        return os.getenv(BrokerEnvVars.BROKER_CLUSTER_SERVER)


class BrokerEnvVars:
    BROKER_SERVER = "BROKER_SERVER"
    BROKER_PORT = "BROKER_PORT"
    BROKER_USERNAME = "BROKER_USERNAME"
    BROKER_PWD = "BROKER_PWD"
    BROKER_CLUSTER_SERVER = "BROKER_CLUSTER_SERVER"



import logging

from hijiki.connection.rabbitmq_connection import ConnectionParameters, RabbitMQConnection


class RabbitMQAdapter:
    def __init__(self, connection_data: ConnectionParameters):
        self.rabbit_connection = None
        self.connection = None
        self.connection_data = connection_data
        self.channel = None

    def connect(self):
        # Cria conexão e canal usando pika
        self.connection = RabbitMQConnection(self.connection_data)
        self.connection.connect()
        self.rabbit_connection = self.connection.connection

    def stop_consuming(self):
        if self.channel.is_open:
            self.channel.close()
        if self.rabbit_connection and self.rabbit_connection.is_open:
            self.rabbit_connection.close()


    def ping(self):
        """Verifica se a conexão está ativa."""
        result = False
        try:
            if self.rabbit_connection and self.rabbit_connection.is_open:
                self.rabbit_connection.process_data_events()
                result = True
        except Exception as e:
            logging.error(f"Erro ao verificar a conexão: {e}")
            result = False
        return result

    def get_channel(self):
        if not self.rabbit_connection or not self.rabbit_connection.is_open:
            self.connect()
        if not self.channel or not self.channel.is_open:
            self.channel = self.rabbit_connection.channel()
            self.channel.basic_qos(prefetch_count=1)
        return self.channel
